//
//  AlbumDataService.m
//
//  Created by Puran Singh on 5/21/12.
//  Copyright (c) 2012 Shutterfly Inc. All rights reserved.
//

#import "AlbumDataService.h"
#import <AssetsLibrary/AssetsLibrary.h>
#import <CoreLocation/CoreLocation.h>

@interface AlbumDataService()
@property(nonatomic, strong) NSMutableArray *mAlbums;
@end

@implementation AlbumDataService

@synthesize mAlbumLoadedBlock;
@synthesize mCompletionBlock;
@synthesize mErrorBlock;
@synthesize mAlbums;
@synthesize mPhotoLoadedBlock;

-(id)init
{
    if(self = [super init]) {
        self.mAlbums = [[NSMutableArray alloc] init];
    }
    return self;
}

- (void)loadAlbums:(PhotoSource)albumSource loadPhotos:(BOOL)loadPhotos
{
    if ([self.mAlbums count] > 0) {
        self.mCompletionBlock(self.mAlbums);
        return;
    }
    ALAssetsLibrary *library = [[ALAssetsLibrary alloc] init]; 
    
    // Group enumerator Block
    void (^assetGroupEnumerator)(ALAssetsGroup *, BOOL *) = ^(ALAssetsGroup *group, BOOL *stop) {
        if (group == nil) {
            if (self.mCompletionBlock != nil) {
                self.mCompletionBlock(self.mAlbums);
            }
            return;
        }
    
        PhotoAlbum *album = [[PhotoAlbum alloc] init];
        album.mAlbumID = [group valueForProperty:@"ALAssetsGroupPropertyPersistentID"];
        album.mAlbumName = [group valueForProperty:@"ALAssetsGroupPropertyName"];
        album.mAlbumURL = [group valueForProperty:@"ALAssetsGroupPropertyURL"];
        album.mCoverPhoto = [UIImage imageWithCGImage:group.posterImage];
        
        [group setAssetsFilter:[ALAssetsFilter allPhotos]];
        album.mPhotoCount = group.numberOfAssets;
        
        album.mSource = IPHONE;
        
        if (loadPhotos) {
            //now enumerate through each group for individual assets
            @autoreleasepool {
                [group enumerateAssetsWithOptions:NSEnumerationConcurrent usingBlock:^(ALAsset *result, NSUInteger index, BOOL *stop) {
                    if(result == nil && index == NSNotFound) {
                        if (self.mAlbumLoadedBlock != nil) {
                            self.mAlbumLoadedBlock(album);
                        }
                        [self.mAlbums addObject:album];
                    } else {
                        Photo *photo = [[Photo alloc] init];
                        photo.mPhotoID =  [result.defaultRepresentation.url description];
                        photo.mSource = album.mSource;
                        photo.mCreatedDate = [result valueForProperty:@"ALAssetPropertyDate"];
                        photo.mLocation = [result valueForProperty:@"ALAssetPropertyLocation"];
                        
                        photo.mThumbnailImageURL = result.defaultRepresentation.url;
                        photo.mFullResImageURL = result.defaultRepresentation.url;
                        
                        [album.mPhotos addObject:photo];
                    }
                }];
            }
        } else {
            if (self.mAlbumLoadedBlock != nil) {
                self.mAlbumLoadedBlock(album);
            }
            [self.mAlbums addObject:album];
        }
    };
    // Group Enumerator Failure Block
    void (^assetGroupEnumberatorFailure)(NSError *) = ^(NSError *error) {    
        if (self.mErrorBlock != nil) {
             self.mErrorBlock([error description]);
        }
    };	
    
    // Enumerate Albums
    [library enumerateGroupsWithTypes:ALAssetsGroupAlbum | ALAssetsGroupLibrary | ALAssetsGroupSavedPhotos | ALAssetsGroupPhotoStream
                           usingBlock:assetGroupEnumerator 
                         failureBlock:assetGroupEnumberatorFailure];
}

- (void)loadPhotos:(NSURL *)albumURL onComplete:(CompletionBlock) complete onPhoto:(PhotoLoadedBlock) photo_ onError:(ErrorBlock) error
{
    self.mPhotoLoadedBlock = photo_;
    self.mCompletionBlock = complete;
    self.mErrorBlock = error;
    
    if ([[albumURL description] isEqualToString:@"favorite://album-list"]) {
        NSMutableDictionary *savedFavorites = [NSKeyedUnarchiver unarchiveObjectWithFile:[self favoritesArchivePath]];
        if (savedFavorites == nil) {
            savedFavorites = [[NSMutableDictionary alloc] init];
        }
        for (Photo *photo in [savedFavorites allValues]) {
            self.mPhotoLoadedBlock(photo);
        }
        self.mCompletionBlock(nil);
        return;
    }
    @autoreleasepool {
        NSMutableArray *photos = [[NSMutableArray alloc] init];
        
        ALAssetsLibrary *library = [[ALAssetsLibrary alloc] init]; 
        [library groupForURL:albumURL resultBlock:^(ALAssetsGroup *group) {
            if (group != nil) {
                [group enumerateAssetsWithOptions:NSEnumerationConcurrent usingBlock:^(ALAsset *result, NSUInteger index, BOOL *stop) {
                    if(result == nil && index == NSNotFound) {                
                        if (self.mCompletionBlock != nil) {
                            self.mCompletionBlock(photos);
                        }
                    } else {
                        Photo *photo = [[Photo alloc] init];
                        photo.mPhotoID =  [result.defaultRepresentation.url description];
                        photo.mCreatedDate = [result valueForProperty:@"ALAssetPropertyDate"];
                        photo.mLocation = [result valueForProperty:@"ALAssetPropertyLocation"];
                        
                        photo.mThumbnailImageURL = result.defaultRepresentation.url;
                        photo.mFullResImageURL = result.defaultRepresentation.url;
                        
                        [photos addObject:photo];
                        self.mPhotoLoadedBlock(photo);
                    }
                }];
            }
        } failureBlock:^(NSError *error) {              
            self.mErrorBlock([error description]);
        }];
    } //end autorelease pool
}

- (void) loadImage:(NSURL *) imageURL imageBlock:(void (^)(UIImage *image))imageBlock
{
    if ([[imageURL absoluteString] hasPrefix:@"assets-library://"]) {
        
        ALAssetsLibraryAssetForURLResultBlock resultblock = ^(ALAsset *myasset) {
            ALAssetRepresentation *rep = [myasset defaultRepresentation];
            CGImageRef iref = [rep fullScreenImage];  
            if (imageBlock) {
                imageBlock ([UIImage imageWithCGImage:iref]);
            }
        };        
        ALAssetsLibraryAccessFailureBlock failureblock  = ^(NSError *myerror) {
            NSLog(@"Cannot get image - %@",[myerror localizedDescription]);
        };
        ALAssetsLibrary* assetslibrary = [[ALAssetsLibrary alloc] init];
        [assetslibrary assetForURL:imageURL 
                       resultBlock:resultblock
                      failureBlock:failureblock];
    }
}

- (BOOL)deleteArchive
{
    self.mAlbums = nil;
    self.mAlbums = [[NSMutableArray alloc] init];
    
    NSError *error;

    if ([self.mAlbums count] > 0) {
        if ([[NSFileManager defaultManager] fileExistsAtPath:[self albumsArchivePath]]) {
            [[NSFileManager defaultManager] removeItemAtPath:[self albumsArchivePath] error:&error];
            if (error) {
                return NO;
            }
        }
    }
    return YES;
}

- (PhotoAlbum *)loadFavoriteAlbum
{
    PhotoAlbum *favoriteAlbum = [[PhotoAlbum alloc] init];
    favoriteAlbum.mAlbumID = @"Favorites";
    favoriteAlbum.mAlbumName = @"Favorites";
    favoriteAlbum.mAlbumURL = [NSURL URLWithString:@"favorite://album-list"];
    favoriteAlbum.mSource = FAVORITES;
    favoriteAlbum.mCoverPhoto = [UIImage imageNamed:@"iconPics.png"];
    return favoriteAlbum;
}

- (void)addToFavorite:(NSString *)photoID photo:(Photo *)photo
{
    NSMutableDictionary *savedFavorites = [NSKeyedUnarchiver unarchiveObjectWithFile:[self favoritesArchivePath]];
    if (savedFavorites == nil) {
        savedFavorites = [[NSMutableDictionary alloc] init];
    }
    
    if ([savedFavorites objectForKey:photoID] == nil) {
        [savedFavorites setObject:photo forKey:photoID];
        BOOL archiveSuccessful = [NSKeyedArchiver archiveRootObject:savedFavorites toFile:[self favoritesArchivePath]];
        if(archiveSuccessful == NO)
            NSLog(@"Couldn't saved to favorites");
    }
}

- (void)removeFromFavorite:(NSString *)photoID
{
    NSMutableDictionary *savedFavorites = [NSKeyedUnarchiver unarchiveObjectWithFile:[self favoritesArchivePath]];
    if ([savedFavorites objectForKey:photoID] != nil) {
        [savedFavorites removeObjectForKey:photoID];
        BOOL archiveSuccessful = [NSKeyedArchiver archiveRootObject:savedFavorites toFile:[self favoritesArchivePath]];
        if(archiveSuccessful == NO)
            NSLog(@"Couldn't saved to favorites");
    }
}


- (NSString *)albumsArchivePath
{
    NSArray *directories = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *directory = [directories objectAtIndex:0];
    
    return [directory stringByAppendingPathComponent:@"iPhoneSavedAlbums.archive"];
}

- (NSString *)favoritesArchivePath
{
    NSArray *directories = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *directory = [directories objectAtIndex:0];
    
    return [directory stringByAppendingPathComponent:@"FavoritesList.archive"];
}
@end
