//
//  AlbumView.m
//  Created by Puran Singh on 1/20/12.
//  Copyright (c) 2012 Shutterfly, Inc. All rights reserved.
//

#import "AlbumView.h"
#import <QuartzCore/QuartzCore.h>
#import "UIView+GMGridViewAdditions.h"
#import "PhotoAlbum.h"
#import "AsyncImageView.h"
#import "UIImage+Resize.h"

@implementation AlbumView

@synthesize mGridView;
@synthesize mSelectedCell;
@synthesize mAlbumList;
@synthesize mDelegate;

- (id)initWithFrame:(CGRect)frame albumList:(NSArray *)albums;
{
    NSInteger spacing = 2;
    self = [super initWithFrame:frame];
    if (self) {
        self.mAlbumList = albums;
        GMGridView *gridView = [[GMGridView alloc] initWithFrame:self.bounds];
        gridView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        gridView.backgroundColor = [UIColor clearColor];

        [self addSubview:gridView];
    
        gridView.style = GMGridViewStyleSwap;
        gridView.itemSpacing = spacing;
        gridView.minEdgeInsets = UIEdgeInsetsMake(spacing, spacing, spacing, spacing);
        gridView.centerGrid = NO;
        gridView.actionDelegate = self;
        gridView.dataSource = self;
        
        self.mGridView = gridView;
    }
    return self;
}

//////////////////////////////////////////////////////////////
#pragma mark GMGridViewDataSource
//////////////////////////////////////////////////////////////

- (NSInteger)numberOfItemsInGMGridView:(GMGridView *)gridView
{
    return [self.mAlbumList count];
}

- (CGSize)GMGridView:(GMGridView *)gridView sizeForItemsInInterfaceOrientation:(UIInterfaceOrientation)orientation
{
    return CGSizeMake(75, 75);
}

- (GMGridViewCell *)GMGridView:(GMGridView *)gridView cellForItemAtIndex:(NSInteger)index
{
    CGSize size = [self GMGridView:gridView sizeForItemsInInterfaceOrientation:[[UIApplication sharedApplication] statusBarOrientation]];
    
    GMGridViewCell *cell = [gridView dequeueReusableCell];
    
    if (!cell) 
    {
        cell = [[GMGridViewCell alloc] init];
        UIView *view = [[UIView alloc] initWithFrame:CGRectMake(0, 0, size.width, size.height)];
        view.layer.masksToBounds = YES;
    
        cell.contentView = view;
    }
    
    [[cell.contentView subviews] makeObjectsPerformSelector:@selector(removeFromSuperview)];

    PhotoAlbum *album = [self.mAlbumList objectAtIndex:index];
    
    AsyncImageView *asyncImageView = [[AsyncImageView alloc] initWithFrame:CGRectMake(0, 0, size.width, size.height)];
    asyncImageView.image = album.mCoverPhoto;
    asyncImageView.contentMode = UIViewContentModeScaleAspectFit;
    [cell.contentView addSubview:asyncImageView];
    asyncImageView.backgroundColor = [UIColor whiteColor];
    
    
    UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(0.0, size.height - 12, size.width, 12)];
    label.text = album.mAlbumName;
    label.textAlignment = UITextAlignmentCenter;
    label.backgroundColor = [UIColor blackColor];
    label.textColor = [UIColor whiteColor];
    label.highlightedTextColor = [UIColor whiteColor];
    label.alpha = 0.8;
    label.font = [UIFont fontWithName:@"Arial" size:8];
    [cell.contentView addSubview:label];
    
    cell.layer.shadowColor = [UIColor blackColor].CGColor;
    cell.layer.shadowOffset = CGSizeMake(3, 4);
    cell.layer.shadowOpacity = 0.7;
    
    return cell;
}

//////////////////////////////////////////////////////////////
#pragma mark GMGridViewActionDelegate
//////////////////////////////////////////////////////////////

- (void)GMGridView:(GMGridView *)gridView didTapOnItemAtIndex:(NSInteger)position
{
    if (self.mSelectedCell != nil) {
       self.mSelectedCell.contentView.layer.shadowColor = [UIColor blackColor].CGColor;
    }
    self.mSelectedCell = [self.mGridView cellForItemAtIndex:position];
    if (self.mSelectedCell.contentView.layer.shadowColor == [UIColor grayColor].CGColor) {
       self.mSelectedCell.contentView.layer.shadowColor = [UIColor blackColor].CGColor;
       [self.mSelectedCell shakeStatus:NO];
    } else {
       self.mSelectedCell.contentView.layer.shadowColor = [UIColor grayColor].CGColor;
    }    
    
    if (self.mDelegate) {
        [self.mDelegate albumView:self didAlbumSelected:[self.mAlbumList objectAtIndex:position]];
    }
}
@end
