
//  Created by Puran Singh on 6/6/12.
//  Copyright (c) 2012 32skills Inc. All rights reserved.
//
#import "AsyncImageView.h"
#import "AppDelegate.h"
#import <AssetsLibrary/AssetsLibrary.h>

@implementation AsyncImageView

@synthesize mImageType, identifier;

- (id)initWithFrame:(CGRect)frame imageType:(DownloadImageType)imageType;
{
    self = [super initWithFrame:frame];
    if (self) {
        self.mImageType = imageType;
    }
    return self;
}

- (void) setImageWithURL:(NSURL *)imageURL onCompletion:(AsyncImageBlock)imageBlock
{
    [self setImageWithURL:imageURL placeholderImage:nil onCompletion:imageBlock];
}

- (void) setImageWithURL:(NSURL *)imageURL placeholderImage:(UIImage *)placeholder onCompletion:(AsyncImageBlock)imageBlock
{
    if (placeholder != nil) {
        self.image = placeholder;
    }
    if ([[imageURL absoluteString] hasPrefix:@"assets-library://"]) {
        ALAssetsLibraryAssetForURLResultBlock resultblock = ^(ALAsset *myasset) {
            ALAssetRepresentation *rep = [myasset defaultRepresentation];
            CGImageRef iref = myasset.thumbnail;
            if (self.mImageType == eDownLoadImageType_LARGE_IMAGE) {
                iref = [rep fullScreenImage];  
            } 
            if (imageBlock) {
                imageBlock([UIImage imageWithCGImage:iref]);
            } else {
                self.image = [UIImage imageWithCGImage:iref];
            }
        };        
        ALAssetsLibraryAccessFailureBlock failureblock  = ^(NSError *myerror) {
            DLog(@"Cannot get image - %@",[myerror localizedDescription]);
        };
        ALAssetsLibrary* assetslibrary = [[ALAssetsLibrary alloc] init];
        [assetslibrary assetForURL:imageURL 
                       resultBlock:resultblock
                      failureBlock:failureblock];
    } else {
        [ApplicationDelegate.imageDownloader imageAtURL:imageURL 
                                           onCompletion:^(UIImage *fetchedImage, NSURL *loadedUrl, BOOL isInCache) {
                                               
                                               if([[imageURL absoluteString] isEqualToString:[loadedUrl absoluteString]]) {
                                                   if (imageBlock) {
                                                       imageBlock (fetchedImage);
                                                   } else {
                                                       self.image = fetchedImage;
                                                   }
                                                   
                                               }
                                           }];
    }
}

@end
