//
//  CarouselView.m
//  pCarousel
//
//  Created by Puran Singh on 7/4/12.
//  Copyright (c) 2012 32Skills. All rights reserved.
//

#import "CarouselView.h"
#import "Photo.h"
#import "AsyncImageView.h"

@interface CarouselView () {
    NSMutableArray *photos;
    iCarousel *carousel;
}

@end

@implementation CarouselView

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self) {
        // Initialization code
        photos = [[NSMutableArray alloc] init]; 
        
        //configure carousel
        carousel = [[iCarousel alloc] initWithFrame:self.frame];
        carousel.type = iCarouselTypeCoverFlow;
        carousel.delegate = self;
        carousel.dataSource = self;
        
        [self addSubview:carousel];
    }
    return self;
}


#pragma mark -
#pragma mark iCarousel methods

- (NSUInteger)numberOfItemsInCarousel:(iCarousel *)carousel
{
    //return the total number of items in the carousel
    return [photos count];
}

- (NSUInteger)numberOfVisibleItemsInCarousel:(iCarousel *)carousel
{
    //return the number of visible carousel items on screen
    //this also affects the appearance of circular-type carousels
    //this value should be <= numberOfItemsInCarousel
    //if you have fewer than about 25 items in your carousel, you don't need
    //to use this method at all (by default it matches numberOfItemsInCarousel)
    return 5;
}

- (UIView *)carousel:(iCarousel *)carousel viewForItemAtIndex:(NSUInteger)index reusingView:(UIView *)view
{
    
    Photo *photo = [photos objectAtIndex:index]; 
    
    AsyncImageView *photoView = nil;
    //create new view if no view is available for recycling
    if (view == nil) {
        view = [[UIView alloc] initWithFrame:self.frame];        
        photoView = [[AsyncImageView alloc] initWithFrame:view.bounds];
        [view addSubview:photoView];
        photoView.contentMode = UIViewContentModeScaleAspectFit;
        photoView.tag = 100;
        photoView.center = view.center;
        photoView.backgroundColor = [UIColor blackColor];
    } else {
        photoView = (AsyncImageView *)[view viewWithTag:100];
    }
    photoView.mImageType = eDownLoadImageType_LARGE_IMAGE;
    [photoView setImageWithURL:photo.mFullResImageURL onCompletion:^(UIImage *fetchedImage) {
        photoView.image = fetchedImage;
    }];
    return view;
}

- (CGFloat)carouselItemWidth:(iCarousel *)carousel
{
    //usually this should be slightly wider than the item views
    return 320;
}

- (CGFloat)carousel:(iCarousel *)carousel itemAlphaForOffset:(CGFloat)offset
{
    //set opacity based on distance from camera
    return 1.0f - fminf(fmaxf(offset, 0.0f), 1.0f);
}

- (CATransform3D)carousel:(iCarousel *)_carousel itemTransformForOffset:(CGFloat)offset baseTransform:(CATransform3D)transform
{
    //implement 'flip3D' style carousel
    transform = CATransform3DRotate(transform, M_PI / 8.0f, 0.0f, 1.0f, 0.0f);
    return CATransform3DTranslate(transform, 0.0f, 0.0f, offset * carousel.itemWidth);
}

- (BOOL)carouselShouldWrap:(iCarousel *)carousel
{
    return YES;
}


- (BOOL)carousel:(iCarousel *)carousel shouldSelectItemAtIndex:(NSInteger)index
{
    return YES;
}

- (void)carousel:(iCarousel *)carousel didSelectItemAtIndex:(NSInteger)index
{
    
}

- (void)addPhoto:(Photo*)photo
{
    [photos addObject:photo];
}

- (void)reloadData
{
    [carousel reloadData];
}
@end
