//
//  FacebookConnectService.m
//
//  Created by Puran Singh on 5/16/12.
//  Copyright (c) 2012 32skills Inc. All rights reserved.
//

#import "FacebookConnectService.h"
#import "PhotoAlbum.h"
#import "Photo.h"

static NSString *ACCOUNT_ARCHIVE = @"facebookAccount.archive";

@implementation FacebookConnectService {
    NSMutableDictionary *facebookAccount;
}

+ (FacebookConnectService *)sharedService {
    static FacebookConnectService *sharedFacebookService = nil;
    if(!sharedFacebookService)
        sharedFacebookService = [[FacebookConnectService alloc] init];
    
    return sharedFacebookService;
}

- (id)initWithAPIClient:(Facebook *)client {
    self = [super init];
    if(!self)
        return self;
    facebookClient = client;
    return self;
}

- (id) init {
    self = [super init];
    if(!self)
        return self;
    
    facebookClient = [[Facebook alloc] initWithAppId:FACEBOOK_CLIENT_ID andDelegate:self];
    [self loadAccount]; 
    return self;
}

- (void)saveAccount {
    BOOL archiveSuccessful = [NSKeyedArchiver archiveRootObject:facebookAccount toFile:[self accountArchivePath]];
    if(archiveSuccessful == NO)
        NSLog(@"Couldn't save facebook account");
}

- (void)loadAccount {
    facebookAccount = [NSKeyedUnarchiver unarchiveObjectWithFile:[self accountArchivePath]];
    if(facebookAccount && [facebookAccount objectForKey:@"accessToken"] && [facebookAccount objectForKey:@"expirationDate"]) {
        facebookClient.accessToken = [facebookAccount objectForKey:@"accessToken"];
        facebookClient.expirationDate = [facebookAccount objectForKey:@"expirationDate"];
    }
}

- (NSString *)accountArchivePath {
    NSArray *directories = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *directory = [directories objectAtIndex:0];
    return [directory stringByAppendingPathComponent:ACCOUNT_ARCHIVE];
}

#pragma mark - Facebook API Calls
/**
 * Make a Graph API Call to get information about the current logged in user.
 */
- (void)apiFQLIMe {
    // Using the "pic" picture since this currently has a maximum width of 100 pixels
    // and since the minimum profile picture size is 180 pixels wide we should be able
    // to get a 100 pixel wide version of the profile picture
    NSMutableDictionary *params = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                   @"SELECT uid, name, pic, email, username, birthday_date FROM user WHERE uid=me()", @"query",
                                   @"me", @"requestType",
                                   nil];
    [facebookClient requestWithMethodName:@"fql.query"
                                andParams:params
                            andHttpMethod:@"POST"
                              andDelegate:self];
}

- (NSString *)albumsArchivePath
{
    NSArray *directories = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *directory = [directories objectAtIndex:0];
    
    return [directory stringByAppendingPathComponent:@"FaebookAlbums.archive"];
}

#pragma mark - instance methods

- (void)albumsWithCompletion:(void (^)(NSMutableArray *albums, NSError *error))block {
    albumsCompleteBlock = [block copy];
    lastRequestCompleteBlock = [block copy];
    
    NSMutableArray *savedAlbums = [NSKeyedUnarchiver unarchiveObjectWithFile:[self albumsArchivePath]];
    if (savedAlbums) {
        albumsCompleteBlock (savedAlbums, nil);
    } else {
        if([facebookClient isSessionValid]) {
            NSMutableDictionary *params = [[NSMutableDictionary alloc] initWithObjectsAndKeys:@"albums", @"requestType", nil];
            [facebookClient requestWithGraphPath:@"me/albums" andParams:params andDelegate:self];
        }
    }
}

- (void)photosFromAlbumWithID:(NSString *)albumID completion:(void (^)(NSArray *photos, NSError *error))block {
    photosCompleteBlock = [block copy];
    lastRequestCompleteBlock = [block copy];
    if([facebookClient isSessionValid]) {
        NSMutableDictionary *params = [[NSMutableDictionary alloc] initWithObjectsAndKeys:@"photos", @"requestType", nil];
        [facebookClient requestWithGraphPath:[NSString stringWithFormat:@"%@/photos", albumID] andParams:params andDelegate:self];
    }
}

- (BOOL)deleteArchive
{
    NSError *error;
    
    if ([[NSFileManager defaultManager] fileExistsAtPath:[self albumsArchivePath]]) {
        [[NSFileManager defaultManager] removeItemAtPath:[self albumsArchivePath] error:&error];
        if (error) {
            return NO;
        }
    }
    return YES;
}

- (void)extendAccessTokenIfNeeded {
    [facebookClient extendAccessTokenIfNeeded];
}

- (BOOL)handleOpenURL:(NSURL *)url {
    return [facebookClient handleOpenURL:url];
}

#pragma SFAuthenticationService methods

- (id)account {
    return facebookAccount;
}

- (void)loginWithCompletion:(void (^)(id account, NSError *error, BOOL canceled))block {
    loginCompleteBlock = [block copy];
    if(![facebookClient isSessionValid]) {
        NSArray *permissions = [[NSArray alloc] initWithObjects:@"user_about_me", 
                                @"user_birthday", 
                                @"user_photos", 
                                @"friends_photos", 
                                nil];
        facebookClient.sessionDelegate = self;
        [facebookClient authorize:permissions];
    }
}

- (BOOL)isLoggedIn {
    return [facebookClient isSessionValid];
}

- (void)logoutWithCompletion:(void (^)(NSError *error))block {
    logoutCompleteBlock = [block copy];
    [facebookClient logout:self];
}

#pragma mark - FBSessionDelegate Methods
/**
 * Called when the user has logged in successfully.
 */
- (void)fbDidLogin {
    [self apiFQLIMe];
}

-(void)fbDidExtendToken:(NSString *)accessToken expiresAt:(NSDate *)expiresAt {
}

/**
 * Called when the user canceled the authorization dialog.
 */
-(void)fbDidNotLogin:(BOOL)cancelled {
    if(loginCompleteBlock)
        loginCompleteBlock(nil, nil, cancelled);
}

/**
 * Called when the request logout has succeeded.
 */
- (void)fbDidLogout {
    if(logoutCompleteBlock)
        logoutCompleteBlock(nil);
}

/**
 * Called when the session has expired.
 */
- (void)fbSessionInvalidated {
  [self fbDidLogout];
}


#pragma mark - FBRequestDelegate Methods
/**
 * Called when the Facebook API request has returned a response. This callback
 * gives you access to the raw response. It's called before
 * (void)request:(FBRequest *)request didLoad:(id)result,
 * which is passed the parsed response object.
 */
- (void)request:(FBRequest *)request didReceiveResponse:(NSURLResponse *)response {
    if ([response isKindOfClass:[NSHTTPURLResponse self]]) {
		NSDictionary *headers = [(NSHTTPURLResponse *)response allHeaderFields];
        for (id key in headers) {
            NSLog(@"Header Value:%@",[headers objectForKey:key]);
        }
	}
}

/**
 * Called when a request returns and its response has been parsed into
 * an object. The resulting object may be a dictionary, an array, a string,
 * or a number, depending on the format of the API response. If you need access
 * to the raw response, use:
 *
 * (void)request:(FBRequest *)request
 *      didReceiveResponse:(NSURLResponse *)response
 */
- (void)request:(FBRequest *)request didLoad:(id)result {
    NSString *requestType = [request.params valueForKey:@"requestType"];
    
    if ([result isKindOfClass:[NSArray class]] && [requestType isEqualToString:@"me"]) {
        result = [result objectAtIndex:0];
    }
    // This callback can be a result of getting the user's basic
    // information or getting the user's permissions.
    if ([requestType isEqualToString:@"me"]) {
        facebookAccount = [self loadFbUserData:result];
        [self saveAccount];
        //we have the basic profile data of the user now do the call back indicating user is 
        //logged in to facebook and have the profile data
        if(loginCompleteBlock)
            loginCompleteBlock(facebookAccount, nil, NO);
    } else if ([requestType isEqualToString:@"albums"]) {
        //fetch facebook album data.
        NSMutableArray *albumList = [[NSMutableArray alloc] init];
    
        NSArray *resultData = [result objectForKey:@"data"];
        NSLog(@"Facebook Albums Data:%@", resultData);
        for (NSDictionary *albumData in resultData) {
            if ([[albumData allKeys] containsObject:@"cover_photo"] == NO) {
                continue;
            }
            PhotoAlbum *facebookAlbum = [[PhotoAlbum alloc] init];
            facebookAlbum.mAlbumID = [albumData objectForKey:@"id"];
            if ([albumData objectForKey:@"name"] != nil) {
                facebookAlbum.mAlbumName = [albumData objectForKey:@"name"];
            } else {
                facebookAlbum.mAlbumName = @"";
            }
            facebookAlbum.mCoverPhotoURL = [NSString stringWithFormat:@"https://graph.facebook.com/%@/picture?access_token=%@", 
                                            [albumData objectForKey:@"cover_photo"], facebookClient.accessToken];
            facebookAlbum.mCreatedDate = [albumData objectForKey:@"created_time"];

            if ([albumData objectForKey:@"count"] != nil) {
                facebookAlbum.mPhotoCount = [[albumData objectForKey:@"count"] integerValue];
            } 
            facebookAlbum.mSource = FACEBOOK;
            
            //add the photo to the list
            [albumList addObject:facebookAlbum];
        }
        BOOL archiveSuccessful = [NSKeyedArchiver archiveRootObject:albumList toFile:[self albumsArchivePath]];
        if(archiveSuccessful == NO)
            NSLog(@"Couldn't save instagram albums");
        if(albumsCompleteBlock)
            albumsCompleteBlock(albumList, nil);
    } else if ([requestType isEqualToString:@"photos"]) {
        NSLog(@"%@", [result description]);
        NSArray *photos = [self photoObjectsFromResult:result];
        if(photosCompleteBlock)
            photosCompleteBlock(photos, nil);
    }
}

- (NSDate *)convertStringToDate:(NSString *)dateString {
    if([dateString isEqual:NSNull.null] || dateString == nil)
        return nil;
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    NSLocale *enUSPOSIXLocale = [[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"];
    [dateFormatter setLocale:enUSPOSIXLocale];
    [dateFormatter setDateFormat:FACEBOOK_BIRTHDAY_DATE_FORMAT];
    return [dateFormatter dateFromString:dateString];
}

- (id)loadFbUserData:(NSDictionary *)result {
    NSLog(@"Facebook User Data:%@", result);
    NSMutableDictionary *newAccount = [[NSMutableDictionary alloc] initWithDictionary:result];
    [newAccount setObject:facebookClient.accessToken forKey:@"accessToken"];
    [newAccount setObject:facebookClient.expirationDate forKey:@"expirationDate"];
    
    return newAccount;
}

/**
 * Called when an error prevents the Facebook API request from completing
 * successfully.
 */
- (void)request:(FBRequest *)request didFailWithError:(NSError *)error {
    NSLog(@"Err message: %@", [[error userInfo] objectForKey:@"error_msg"]);
    NSLog(@"Err code: %d", [error code]);
    if(lastRequestCompleteBlock)
        lastRequestCompleteBlock(nil, error);
}

#pragma mark - Private methods
- (Photo *)photoObjectFromData:(NSDictionary *)photoData {
    Photo *photo = [[Photo alloc] init];
    
    photo.mPhotoID = [photoData objectForKey:@"id"];
    photo.mSource = FACEBOOK;
    
    if ([[photoData allKeys] containsObject:@"name"] == YES) {
        photo.mPhotoTitle = [photoData objectForKey:@"name"];
    }
    photo.mThumbnailImageURL = [NSURL URLWithString:[photoData objectForKey:@"picture"]];
    photo.mFullResImageURL = [NSURL URLWithString:[photoData objectForKey:@"source"]];
    
    //date string containing ISO-8601 date-time
    photo.mCreatedDate = [photoData objectForKey:@"created_time"];

    return photo;
}

- (NSArray *)photoObjectsFromResult:(id)result {
    NSMutableArray *photoList = [[NSMutableArray alloc] init];
    NSArray *resultData = [result objectForKey:@"data"];
    for (NSDictionary *photoData in resultData) {
        [photoList addObject:[self photoObjectFromData:photoData]];
    }
    return [[NSArray alloc] initWithArray:photoList];
}

@end
