//
//  ViewController.m
//  pCarousel
//
//  Created by Puran Singh on 6/8/12.
//  Copyright (c) 2012 32skills Inc. All rights reserved.
#import "HomeViewController.h"
#import "AlbumDataService.h"
#import "PhotoAlbum.h"
#import "PhotoGridViewController.h"
#import "LoadingView.h"
#import "PhotoStore.h"
#import "MenuViewController.h"
#import "InstagramConnectService.h"
#import "FacebookConnectService.h"
#import "SettingsButtonItem.h"
#import <MobileCoreServices/MobileCoreServices.h>
#import "AsyncImageView.h"
#import "UIImage+Resize.h"
#import "SVProgressHUD.h"
#import <DropboxSDK/DropboxSDK.h>


NSString *const kFacebook = @"facebook";
NSString *const kInstagram = @"instagram";
NSString *const kIPhone = @"iPhone";
NSString *const kDropBox = @"dropbox";
NSString *const kFavorites = @"favorites";

@interface HomeViewController () {
    MenuViewController *menu;
    UIActionSheet *actionSheet;
    CGPoint carouselCenter;
    UIView *newPhotoView;
}
@property (nonatomic, strong) NSMutableArray            *mAlbums;
@property (nonatomic, strong) LoadingView               *mBusyIndicator;
@property (nonatomic, strong) AlbumDataService          *mAlbumDataService;
@property (nonatomic, assign) BOOL                      busy;

- (void) showMenu:(id)sender;
- (void) launchPhotoPicker:(NSString *)source;
@end

@implementation HomeViewController
@synthesize cameraButton;
@synthesize carousel;
@synthesize menuView;
@synthesize mBusyIndicator;
@synthesize mAlbumDataService;
@synthesize mAlbums;
@synthesize busy;

- (void)didReceiveMemoryWarning
{
    NSLog(@"Memory Error");
}

#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // someplace where you create the UINavigationController
    if ([self.navigationController.navigationBar respondsToSelector:@selector(setBackgroundImage:forBarMetrics:)] )
    {
        UIImage *image = [UIImage imageNamed:@"backgroundNavigationBar.png"];
        [self.navigationController.navigationBar setBackgroundImage:image forBarMetrics:UIBarMetricsDefault];
    }
        
    NSArray* nibViews = [[NSBundle mainBundle] loadNibNamed:@"LoadingView"
                                                      owner:self
                                                    options:nil];
    
    self.mBusyIndicator = [ nibViews objectAtIndex:0];
    self.mBusyIndicator.center = [self.mBusyIndicator convertPoint:self.carousel.center toView:self.view];
    //[self.view addSubview:self.mBusyIndicator];
    
    UIBarButtonItem *menuButton = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"menu.png"]
                                                                   style:UIBarButtonItemStyleBordered
                                                                  target:self
                                                                  action:@selector(showMenu:)];
    
    self.navigationItem.leftBarButtonItem = menuButton;
    
    UIBarButtonItem *refresh = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh
                                                                             target:self
                                                                             action:@selector(refreshAlbums)];
    
    SettingsButtonItem *settingsButton = [[SettingsButtonItem alloc] initWithTarget:self
                                                                             action:@selector(showSettings:)];
    
    self.navigationItem.rightBarButtonItems = [[NSArray alloc] initWithObjects:refresh, settingsButton, nil];
    
    
    //Initialize Toolbar
    
    UIBarButtonItem *cancelBtn = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel
                                                                               target:self
                                                                               action:@selector(cancelNewPhoto)];
    
    UIBarButtonItem *flexible = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    UIBarButtonItem *actionButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAction
                                                                                  target:self action:@selector(newPhotoAction)];
    
    self.toolbarItems = [NSArray arrayWithObjects:cancelBtn, flexible, actionButton, nil];

    
    self.mAlbumDataService = [[AlbumDataService alloc] init];
    self.mAlbums = [[NSMutableArray alloc] init];
    
    UITextView *titleView = [[UITextView alloc] initWithFrame:CGRectMake(0, 0, 150, 40)];
    titleView.text = @"Albums";
    titleView.textAlignment = UITextAlignmentCenter;
    titleView.backgroundColor = [UIColor clearColor];
    titleView.textColor = [UIColor whiteColor];
    titleView.font = [UIFont fontWithName:@"American Typewriter"  size:19];
    titleView.editable = NO; 
    
    self.navigationItem.titleView = titleView;
    //configure carousel
    carousel.type = iCarouselTypeCoverFlow;

    //initialize menu
    self->menu = [[MenuViewController alloc] init];
    [self addChildViewController:self->menu];
    [self.view addSubview:menu.view];
    self->menu.view.center = CGPointMake(80, 215.0f);
    self->menu.delegate = self;
    [self.view sendSubviewToBack:self->menu.view];
    
    UIView *texturedBackgroundView = [[UIView alloc] initWithFrame:self.carousel.bounds];
    [texturedBackgroundView setBackgroundColor:[UIColor colorWithPatternImage:[UIImage imageNamed:@"backgroundLeather.png"]]];
    [self.carousel addSubview:texturedBackgroundView];
    [self.carousel sendSubviewToBack:texturedBackgroundView];
    carouselCenter = self.carousel.center;
}

- (void)viewDidUnload
{
    [self setCarousel:nil];
    [self setMBusyIndicator:nil];
    [self setMenuView:nil];
    [self setCameraButton:nil];
    [self setMAlbums:nil];
    [self setMAlbumDataService:nil];

    self->menu = nil;
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated
{
    if ([[PhotoStore store].selectedSources count] == 0)
    {
        [self->menu markSelected:kIPhone];
    }
    else
    {
        if ([[PhotoStore store].selectedSources containsObject:kIPhone])
        {
            [self->menu markSelected:kIPhone];
        }
        if ([[PhotoStore store].selectedSources containsObject:kFacebook])
        {
            [self->menu markSelected:kFacebook];
        }
        if ([[PhotoStore store].selectedSources containsObject:kInstagram])
        {
            [self->menu markSelected:kInstagram];
        }
        if ([[PhotoStore store].selectedSources containsObject:kDropBox])
        {
            [self->menu markSelected:kDropBox];
        }
        if ([[PhotoStore store].selectedSources containsObject:kFavorites])
        {
            [self->menu markSelected:kFavorites];
        }

    }
    [super viewWillAppear:animated];
}


- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    if ([self.mAlbums count] > 0)
    {
        return;
    }
    [self loadAllAlbums];
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if (UIInterfaceOrientationIsPortrait(interfaceOrientation))
    {
        return YES;
    }
    return NO;
}

- (void)cancelNewPhoto
{
    CGPoint center = self->newPhotoView.center;
    
    self.navigationController.navigationBar.translucent=NO;
    self.navigationController.toolbar.translucent = YES;
    
    [self.navigationController setToolbarHidden:YES animated:NO];
    [self.navigationController setNavigationBarHidden:NO animated:NO];
    
    [UIView animateWithDuration:0.20 animations:^{
        self->newPhotoView.center = CGPointMake(center.x, center.y * 3);
    } completion:^(BOOL finished) {
        [self->newPhotoView removeFromSuperview];
        self->newPhotoView = nil;
        self.carousel.userInteractionEnabled = YES;
    }];
}

- (void)newPhotoAction
{
    self->actionSheet = [[UIActionSheet alloc] initWithTitle:@"" delegate:self cancelButtonTitle:nil destructiveButtonTitle:@"Cancel" otherButtonTitles:@"Save to Dropbox", @"Post to Facebook", @"Save to Photo Library", nil];
    [self->actionSheet showInView:self.view];
}

#pragma mark -
#pragma mark iCarousel methods

- (NSUInteger)numberOfItemsInCarousel:(iCarousel *)carousel
{
    //return the total number of items in the carousel
    return [self.mAlbums count];
}

- (NSUInteger)numberOfVisibleItemsInCarousel:(iCarousel *)carousel
{
    //return the number of visible carousel items on screen
    //this also affects the appearance of circular-type carousels
    //this value should be <= numberOfItemsInCarousel
    //if you have fewer than about 25 items in your carousel, you don't need
    //to use this method at all (by default it matches numberOfItemsInCarousel)
    if ([self.mAlbums count] < 25) {
        return [self.mAlbums count];
    }
    return 5;
}

- (UIView *)carousel:(iCarousel *)carousel viewForItemAtIndex:(NSUInteger)index reusingView:(UIView *)view
{   
    PhotoAlbum *album = [self.mAlbums objectAtIndex:index];
    
    //create new view if no view is available for recycling
    AsyncImageView *coverPhotoView = nil;
    UITextView *titleView = nil;
    UIImageView *sourceLabel;
    if (view == nil)
    {
        view = [[UIView alloc] initWithFrame:CGRectMake(0., 0., 240, 350)];
        UIImageView *bgView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"page.png"]];
        [view addSubview:bgView];
        bgView.frame = view.bounds;
        
        coverPhotoView = [[AsyncImageView alloc] init];
        coverPhotoView.contentMode = UIViewContentModeScaleAspectFit;
        coverPhotoView.tag = 10;
        coverPhotoView.center = bgView.center;
        coverPhotoView.layer.masksToBounds = YES;
        [view addSubview:coverPhotoView];
        coverPhotoView.backgroundColor = [UIColor whiteColor];
        
        CGRect textFrame = CGRectMake(coverPhotoView.bounds.origin.x + 20, coverPhotoView.bounds.origin.y + 50, 200, 30.0);
        titleView = [[UITextView alloc] initWithFrame:textFrame];
        titleView.text = album.mAlbumName;
        titleView.textAlignment = UITextAlignmentCenter;
        titleView.backgroundColor = [UIColor clearColor];
        titleView.textColor = [UIColor whiteColor];
        titleView.font = [UIFont fontWithName:@"American Typewriter" size:14];
        titleView.editable = NO; 
        titleView.layer.borderWidth = 1.0;
        titleView.layer.borderColor = [UIColor lightTextColor].CGColor;
        titleView.layer.backgroundColor = [UIColor darkTextColor].CGColor;
        titleView.alpha = 0.7;
        titleView.layer.cornerRadius = 5.0f;
        titleView.tag = 1;
        titleView.scrollEnabled = NO;
        
        [view addSubview:titleView];
        
        sourceLabel = [[UIImageView alloc] initWithFrame:CGRectMake(view.bounds.size.width - 55,
                                                                        view.bounds.size.height - 120,
                                                                        30,
                                                                        30)];
        [view addSubview:sourceLabel];
        
    } else {
        coverPhotoView = (AsyncImageView *)[view viewWithTag:10];
        titleView = (UITextView *)[view viewWithTag:1];
        sourceLabel = (UIImageView *)[view viewWithTag:2];
    }
    if (album.mSource == FACEBOOK) {
        sourceLabel.image = [UIImage imageNamed:@"f_logo.png"];
    } else if (album.mSource == INSTAGRAM) {
        sourceLabel.image = [UIImage imageNamed:@"instagramIcon_bigger.png"];
    } else {
    }
    coverPhotoView.bounds = CGRectMake(0., 0., album.mCoverPhoto.size.width, album.mCoverPhoto.size.height);
    coverPhotoView.bounds = CGRectMake(0.0f, 0.0f, 80.0f, 80.0f);
    if (album.mSource == INSTAGRAM || album.mSource == FACEBOOK) {
        [coverPhotoView setImageWithURL:[NSURL URLWithString:album.mCoverPhotoURL]
                       placeholderImage:[UIImage imageNamed:@"placeholder_icon.png"]
                           onCompletion:^(UIImage *fetchedImage) {
                               coverPhotoView.image = [fetchedImage resizedImage:CGSizeMake(80.0f, 80.0f) interpolationQuality:kCGInterpolationDefault];
        }];
    } else {
        coverPhotoView.image = album.mCoverPhoto;
    }
    titleView.text = album.mAlbumName;
    
    coverPhotoView.layer.borderColor = [UIColor lightGrayColor].CGColor;
    coverPhotoView.layer.borderWidth = 3.0;
    
    return view;
}

- (CGFloat)carouselItemWidth:(iCarousel *)carousel
{
    //usually this should be slightly wider than the item views
    return 300;
}

- (CGFloat)carousel:(iCarousel *)carousel itemAlphaForOffset:(CGFloat)offset
{
    //set opacity based on distance from camera
    return 1.0f - fminf(fmaxf(offset, 0.0f), 1.0f);
}

- (CATransform3D)carousel:(iCarousel *)_carousel itemTransformForOffset:(CGFloat)offset baseTransform:(CATransform3D)transform
{
    //implement 'flip3D' style carousel
    transform = CATransform3DRotate(transform, M_PI / 8.0f, 0.0f, 1.0f, 0.0f);
    return CATransform3DTranslate(transform, 0.0f, 0.0f, offset * carousel.itemWidth);
}

- (BOOL)carouselShouldWrap:(iCarousel *)carousel
{
    return NO;
}


- (BOOL)carousel:(iCarousel *)carousel shouldSelectItemAtIndex:(NSInteger)index
{
    return YES;
}

- (void)carousel:(iCarousel *)carousel didSelectItemAtIndex:(NSInteger)index
{
    PhotoGridViewController *pCarousel = [[PhotoGridViewController alloc] init];
    [[PhotoStore store] setNewActiveAlbum:[self.mAlbums objectAtIndex:index]];
    [self.navigationController pushViewController:pCarousel animated:YES];
}

- (void) launchPhotoPicker:(NSString *)source
{
    if (([UIImagePickerController isSourceTypeAvailable:
          UIImagePickerControllerSourceTypeCamera] == NO) && [source isEqualToString:@"camera"]) {
        UIAlertView *error = [[UIAlertView alloc] initWithTitle:@"Error" message:@"Camera not available" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [error show];
    } else {
        UIImagePickerController *cameraUI = [[UIImagePickerController alloc] init];
        if ([source isEqualToString:@"camera"]) {
            cameraUI.sourceType = UIImagePickerControllerSourceTypeCamera;
            // Displays a control that allows the user to choose picture or
            // movie capture, if both are available:
            cameraUI.mediaTypes =
            [UIImagePickerController availableMediaTypesForSourceType:
             UIImagePickerControllerSourceTypeCamera];
        } else {
            cameraUI.sourceType = UIImagePickerControllerSourceTypePhotoLibrary | UIImagePickerControllerSourceTypeSavedPhotosAlbum;
            // Displays a control that allows the user to choose picture or
            // movie capture, if both are available:
            cameraUI.mediaTypes =
            [UIImagePickerController availableMediaTypesForSourceType:
             UIImagePickerControllerSourceTypePhotoLibrary | UIImagePickerControllerSourceTypeSavedPhotosAlbum];
        }
        // Hides the controls for moving & scaling pictures, or for
        // trimming movies. To instead show the controls, use YES.
        cameraUI.allowsEditing = NO;
        cameraUI.showsCameraControls = YES;
        
        cameraUI.delegate = self;
        
        [self presentViewController:cameraUI animated:YES completion:nil];
    }
}

#pragma mark - camera delegate

// For responding to the user tapping Cancel.
- (void) imagePickerControllerDidCancel: (UIImagePickerController *) picker {
    [picker dismissModalViewControllerAnimated:YES];
}

// For responding to the user accepting a newly-captured picture or movie
- (void) imagePickerController: (UIImagePickerController *) picker
 didFinishPickingMediaWithInfo: (NSDictionary *) info {
    
    [SVProgressHUD showWithStatus:@"Loading Editor..."];
    NSString *mediaType = [info objectForKey: UIImagePickerControllerMediaType];
    UIImage *originalImage;
    
    // Handle a still image capture
    if (CFStringCompare ((__bridge CFStringRef) mediaType, kUTTypeImage, 0) == kCFCompareEqualTo) {

        originalImage = (UIImage *) [info objectForKey:
                                     UIImagePickerControllerOriginalImage];
        
        // Save the new image (original or edited) to the Camera Roll
        //UIImageWriteToSavedPhotosAlbum (imageToSave, nil, nil , nil);
    }
    [picker dismissViewControllerAnimated:YES completion:^{
        [self loadPhotoEditor:originalImage];
    }];
    picker = nil;
}

#pragma mark - PhotoEditor Delegate

- (void)photoEditor:(AFPhotoEditorController *)editor finishedWithImage:(UIImage *)image
{
    [self dismissModalViewControllerAnimated:YES];
    self.carousel.userInteractionEnabled = NO;
    self.navigationController.navigationBar.translucent=YES;
    self.navigationController.toolbar.translucent = YES;
    [self.navigationController setToolbarHidden:NO];
    [self.navigationController setNavigationBarHidden:YES];
    
    [self.navigationController.toolbar setBackgroundImage:[UIImage imageNamed:@"backgroundNavigationBar.png"]
                                       forToolbarPosition:UIToolbarPositionBottom
                                               barMetrics:UIBarMetricsDefault];
    
    CGFloat aspectRatio = image.size.width / image.size.height;
    CGFloat newWidth = 320.0f;
    CGFloat newHeight = newWidth / aspectRatio;
    
    self->newPhotoView = [[UIView alloc] initWithFrame:self.view.bounds];
    [self.view addSubview:self->newPhotoView];
    self->newPhotoView.backgroundColor = [UIColor blackColor];
    
    UIImageView *photoView = [[UIImageView alloc] initWithFrame:self.view.bounds];
    photoView.image = [image resizedImage:CGSizeMake(newWidth, newHeight) interpolationQuality:kCGInterpolationMedium];
    [self->newPhotoView addSubview:photoView];
    photoView.center = self->newPhotoView.center;
    photoView.contentMode = UIViewContentModeScaleAspectFit;
}

- (void)photoEditorCanceled:(AFPhotoEditorController *)editor
{
    [self dismissModalViewControllerAnimated:YES];
}

- (void)loadPhotoEditor:(UIImage *)image
{
    AFPhotoEditorController *editor = [[AFPhotoEditorController alloc] initWithImage:image];
    
    [editor setDelegate:self];
    editor.title = @"Edit Photo";
    UIColor *backgroundColor = [UIColor colorWithRed:122.0f/255.0f green:204.0f/255.0f blue:41.0f/255.0f alpha:1.0];
    UIColor *foregroundColor = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:1.0];
    
    AFPhotoEditorStyle *style = [editor style];
    [style setTopBarBackgroundColor:backgroundColor];
    [style setTopBarTextColor:foregroundColor];

    [SVProgressHUD dismiss];
    
    [self presentViewController:editor animated:YES completion:nil];
}

- (void)dismissBusy
{
    if (self.busy == NO) {
        [SVProgressHUD dismiss];
    }
}

- (void)loadAllAlbums
{
    [SVProgressHUD showWithStatus:@"Loading Albums..." networkIndicator:YES];
    self.busy = YES;
    NSBlockOperation *loadJob = [NSBlockOperation blockOperationWithBlock:^{
        if ([[PhotoStore store].selectedSources count] == 0) {
            [self loadAlbums:YES];
        } else {
            if ([[PhotoStore store].selectedSources containsObject:kIPhone]) {
                [self loadAlbums:YES];
            }
            if ([[PhotoStore store].selectedSources containsObject:kFavorites]) {
                [self loadFavoriteAlbum];
            }
            if ([[PhotoStore store].selectedSources containsObject:kFacebook]) {
                [self facebookAlbums:YES];
            }
            if ([[PhotoStore store].selectedSources containsObject:kInstagram]) {
                [self instagramAlbums:YES];
            }
        }
    }];
    
    [loadJob setCompletionBlock:^{
        [self dismissBusy];
    }];
    // put it in the queue for execution
    [[NSOperationQueue mainQueue] addOperation:loadJob];
}

- (void)refreshAlbums
{
    [self.carousel scrollToItemAtIndex:0 animated:YES];
    
    [SVProgressHUD showWithStatus:@"Refreshing albums..." networkIndicator:YES];
    
    NSBlockOperation *refreshJob = [NSBlockOperation blockOperationWithBlock:^{
        if ([[PhotoStore store].selectedSources containsObject:kIPhone]) {
            if ([self.mAlbumDataService deleteArchive] == YES) {
                [self filterAlbums:IPHONE];
                [self loadAlbums:YES];
            }
        }
        if ([[PhotoStore store].selectedSources containsObject:kFacebook]) {
            if ([[FacebookConnectService sharedService] deleteArchive] == YES) {
                [self filterAlbums:FACEBOOK];
                [self facebookAlbums:YES];
            }
        }
        if ([[PhotoStore store].selectedSources containsObject:kInstagram]) {
            if ([[InstagramConnectService sharedService] deleteArchive] == YES) {
                [self filterAlbums:INSTAGRAM];
                [self instagramAlbums:YES];
            }
        }
    }];
    
    [refreshJob setCompletionBlock:^{
        [SVProgressHUD dismiss];
    }]; 
    // put it in the queue for execution
    [[NSOperationQueue mainQueue] addOperation:refreshJob];
}

- (void)closeMenu
{
    [UIView animateWithDuration:0.30 animations:^{
        self.carousel.center = carouselCenter;
    } completion:^(BOOL finished) {
        self->menu.open = NO;
        self.carousel.userInteractionEnabled = YES;
    }];
}

- (void) showMenu:(id)sender
{
    if (self->menu.open == YES) {
        [self closeMenu];
    } else {
        self->menu.open = YES;
        self.carousel.userInteractionEnabled = NO;
        self.carousel.layer.shadowOffset = CGSizeMake(-6.0f, 0.0f);
        self.carousel.layer.shadowOpacity = 1.0f;
        self.carousel.layer.shadowColor = [UIColor blackColor].CGColor;
        carouselCenter = self.carousel.center;
        [UIView animateWithDuration:0.20 animations:^{
            self.carousel.center = CGPointMake(self.carousel.center.x + 150.0f, self.carousel.center.y);
        } completion:^(BOOL finished) {
            [self.carousel scrollToItemAtIndex:0 animated:YES];    
        }];    
        [self didMoveToParentViewController:menu];
    }
}

- (IBAction)cameraButtonClicked:(id)sender {
    [self launchPhotoPicker:@"camera"];
}

#pragma mark - UIActionSheetDelegate methods

// Called when a button is clicked. The view will be automatically dismissed after this call returns
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    NSString *choice = [self->actionSheet buttonTitleAtIndex:buttonIndex];
    if (buttonIndex == [self->actionSheet destructiveButtonIndex]) {
        //Cancel
    } else if ([choice isEqualToString:@"Take New Picture"]){
        [self launchPhotoPicker:@"camera"];
    } else if ([choice isEqualToString:@"Choose From Library"]){
        [self launchPhotoPicker:@"library"];
    }
}

// Called when we cancel a view (eg. the user clicks the Home button). This is not called when the user clicks the cancel button.
// If not defined in the delegate, we simulate a click in the cancel button
- (void)actionSheetCancel:(UIActionSheet *)actionSheet
{
    NSLog(@"Canceled");
}

- (void)loadAlbums:(BOOL)refresh
{
    if (refresh == YES) {
        __weak typeof (self) myself = self;
        self.mAlbumDataService.mCompletionBlock = ^(NSArray *albums) {
            self.busy = NO;
            [myself dismissBusy];
            [myself.mAlbums addObjectsFromArray:[[NSMutableArray alloc] initWithArray:albums]];
            [myself.carousel reloadData];
        };
    } else {
        self.busy = YES;
        [SVProgressHUD showWithStatus:@"Loading Albums..." networkIndicator:NO];
        //refresh albums view after each album is loaded
        __weak typeof (self) myself = self;
        self.mAlbumDataService.mCompletionBlock = ^(NSArray *albums) {
            self.busy = NO;
            [myself dismissBusy];
            [myself.mAlbums addObjectsFromArray:[[NSMutableArray alloc] initWithArray:albums]];
            [myself.carousel reloadData];
        };
    }
    [self.mAlbumDataService loadAlbums:IPHONE loadPhotos:NO];
}

- (void)dropBoxAlbums:(BOOL)refresh
{
    if (![[DBSession sharedSession] isLinked]) {
		[[DBSession sharedSession] linkFromController:self];
    } else {
        [[DBSession sharedSession] unlinkAll];
        [[[UIAlertView alloc]
           initWithTitle:@"Account Unlinked!" message:@"Your dropbox account has been unlinked"
           delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil]
         show];
    }
}

- (void)instagramAlbums:(BOOL)refresh
{
    if (refresh == YES) {
        __weak typeof (self) myself = self;
        [[InstagramConnectService sharedService] albums:^(NSMutableArray *albums, NSError *error) {
            self.busy = NO;
            [myself dismissBusy];
            [myself.mAlbums addObject:[albums objectAtIndex:0]];
            [myself.carousel reloadData];
        }];
    } else {
        self.busy = YES;
        [SVProgressHUD showWithStatus:@"Loading Instagram Albums..." networkIndicator:YES];
        __weak typeof (self) myself = self;
        [[InstagramConnectService sharedService] albums:^(NSMutableArray *albums, NSError *error) {
            [myself.mAlbums addObject:[albums objectAtIndex:0]];
            self.busy = NO;
            [myself dismissBusy];
            
            [myself.carousel reloadData];
            [myself.carousel scrollToItemAtIndex:[self.mAlbums count] animated:YES];
        }];
    }
}

- (void)loadFavoriteAlbum
{
    [self.mAlbums addObject:[self.mAlbumDataService loadFavoriteAlbum]];
    [self.carousel reloadData];
    [self.carousel scrollToItemAtIndex:[self.mAlbums count] animated:YES];
}

- (void)facebookAlbums:(BOOL)refresh
{
    if (refresh == YES) {
        __weak typeof (self) myself = self;
        [[FacebookConnectService sharedService] albumsWithCompletion:^(NSMutableArray *albums, NSError *error) {
            self.busy = NO;
            [myself dismissBusy];
            [myself.mAlbums addObjectsFromArray:albums];
            [myself.carousel reloadData];
        }];
    } else {
        [SVProgressHUD showWithStatus:@"Loading Facebook Albums..." networkIndicator:YES];
        self.busy = YES;
         __weak typeof (self) myself = self;
        [[FacebookConnectService sharedService] albumsWithCompletion:^(NSMutableArray *albums, NSError *error) {
            self.busy = NO;
            [myself.mAlbums addObjectsFromArray:albums];
            [myself dismissBusy];
            
            [myself.carousel reloadData];
            [myself.carousel scrollToItemAtIndex:[self.mAlbums count] animated:YES];
        }];
    }
}

- (void)filterAlbums:(PhotoSource)source
{
    for (PhotoAlbum *album in [self.mAlbums copy]) {
        if (album.mSource == source) {
            [self.mAlbums removeObject:album];
        }
    }
    [self.carousel reloadData];
}

- (void)menuSelected:(NSString *)action enable:(BOOL)enable sender:(UIButton *)sender
{
    [self closeMenu];
    
    if ([action isEqualToString:kInstagram] && enable == YES)
    {
        [[PhotoStore store].selectedSources addObject:kInstagram];
        if ([[InstagramConnectService sharedService] isLoggedIn] == NO)
        {
            [[InstagramConnectService sharedService] loginWithCompletion:^(id account, NSError *error, BOOL canceled) {
                    
                [self instagramAlbums:NO];
               
            }];
        }
        else {
            [self instagramAlbums:NO];
        }
    }
    else if ([action isEqualToString:kInstagram] && enable == NO)
    {
        [[PhotoStore store].selectedSources removeObject:kInstagram];
        [self filterAlbums:INSTAGRAM];
    }
    else if ([action isEqualToString:kFacebook] && enable == YES)
    {
        [[PhotoStore store].selectedSources addObject:kFacebook];
        if ([[FacebookConnectService sharedService] isLoggedIn] == NO)
        {
            [[FacebookConnectService sharedService] loginWithCompletion:^(id account, NSError *error, BOOL canceled) {
                [self facebookAlbums:NO];
            }];
        }
        else
        {
            [self facebookAlbums:NO];
        }
    }
    else if ([action isEqualToString:kFacebook] && enable == NO)
    {
        [[PhotoStore store].selectedSources removeObject:kFacebook];
        [self filterAlbums:FACEBOOK];
    }
    else if ([action isEqualToString:kIPhone] && enable == YES)
    {
        [[PhotoStore store].selectedSources addObject:kIPhone];
        [self loadAlbums:NO];
    }
    else if ([action isEqualToString:kIPhone] && enable == NO)
    {
        [[PhotoStore store].selectedSources removeObject:kIPhone];
        [self filterAlbums:IPHONE];
    }
    else if ([action isEqualToString:kDropBox] && enable == YES)
    {
        [[PhotoStore store].selectedSources addObject:kDropBox];
        [self dropBoxAlbums:NO];
    }
    else if ([action isEqualToString:kDropBox] && enable == NO)
    {
        [[PhotoStore store].selectedSources removeObject:kDropBox];
        [self filterAlbums:DROPBOX];
    }
    else if ([action isEqualToString:kFavorites] && enable == YES)
    {
        [[PhotoStore store].selectedSources addObject:kFavorites];
        [self loadFavoriteAlbum];
    }
    else if ([action isEqualToString:kFavorites] && enable == NO)
    {
        [[PhotoStore store].selectedSources removeObject:kFavorites];
        [self filterAlbums:FAVORITES];
    }
}

@end