
//spacemonkey
#import "InstagramAPIClient.h"


#define RESPONSE_URL (@"photoplus://instagram")
#define API_SCOPE @"basic"
#define BASE_URL @"https://api.instagram.com/v1"


NSString* ACTION_GET_USER_MEDIA_RECENT      = @"ACTION_GET_USER_MEDIA_RECENT";
NSString* ACTION_GET_USER_MEDIA_LIKED       = @"ACTION_GET_USER_MEDIA_LIKED";
NSString* ACTION_GET_USER_FEED              = @"ACTION_GET_USER_FEED";
NSString* ACTION_GET_MEDIA_POPULAR          = @"ACTION_GET_MEDIA_POPULAR";
NSString* ACTION_GET_TAG_MEDIA_RECENT       = @"ACTION_GET_TAG_MEDIA_RECENT";
NSString* ACTION_GET_USER_SELF              = @"ACTION_GET_USER_SELF";


static NSString* USER_INFO_KEY_ACTION       = @"action";
static NSString* USER_INFO_KEY_IMG_ID       = @"imgId";
static NSString* USER_INFO_KEY_USER_INFO    = @"userInfo";

@interface InstagramAPIClient () 
- (NSString*) getResponseURL;
- (NSString*) getLoginURL;
- (NSString*) getLogoutURL;
- (NSDictionary*)parseURLParams:(NSString *)query;
@end

@implementation InstagramAPIClient

@synthesize mApplicationId;
@synthesize mAccessToken;
@synthesize mSessionDelegate;
@synthesize mRequestDelegate;


- (id)initWithAppId:(NSString *)appId
        andDelegate:(id<InstagramAPISessionDelegate>)delegate
{
    self = [super initWithHostName:nil customHeaderFields:nil];
    if (self) {
        self.mApplicationId = appId;
        self.mSessionDelegate = delegate;
    }
    return self;
}

- (NSString*) getResponseURL {
    return RESPONSE_URL;
}

- (NSString*) getLoginURL {
    NSString* responseUrl = [[self getResponseURL] stringByAddingPercentEscapesUsingEncoding:NSASCIIStringEncoding];
    NSString* reqURL = [NSString stringWithFormat:@"https://api.instagram.com/oauth/authorize/?client_id=%@&redirect_uri=%@&response_type=token&scope=" 
                        API_SCOPE "&display=touch", 
                        INSTAGRAM_CLIENT_ID, responseUrl];
    
    return reqURL;
}

- (NSString*) getLogoutURL {
    return @"https://instagram.com/accounts/logout/";
}

#pragma public methods

- (void) login {
    if (![self isSessionValid]) {
        NSString* reqUrl = [self getLoginURL];
        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:reqUrl]];   
    }
}

- (void) logout {
    if ([self isSessionValid]) {
        //do logout here
    }
}

- (BOOL) isSessionValid {
    return (self.mAccessToken != nil);
}

- (NSDictionary*)parseURLParams:(NSString *)query 
{
	NSArray *pairs = [query componentsSeparatedByString:@"&"];
	NSMutableDictionary *params = [[NSMutableDictionary alloc] init];
	for (NSString *pair in pairs) {
		NSArray *kv = [pair componentsSeparatedByString:@"="];
		NSString *val =
        [[kv objectAtIndex:1]
         stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        
		[params setObject:val forKey:[kv objectAtIndex:0]];
	}
    return params;
}

- (BOOL) handleOpenURL:(NSURL*)url {
    /*
    STEP TWO: RECEIVE THE REDIRECT FROM INSTAGRAM
    Once a user successfully authenticates and authorizes your application, we will redirect the user to your redirect_uri with a code parameter that you'll use in step three.
    
    http://your-redirect-uri?code=CODE
    If this request for approval is denied, then we will redirect the user to your redirect_uri with the following parameters:
    
    error: access_denied
    error_reason: user_denied
    error_description: The user denied your request
    http://your-redirect-uri?error=access_denied&error_reason=user_denied&error_description=The+user+denied+your+request
    It is your responsibility to fail gracefully in this situation and display a corresponding error message to your user.
    */
    
    NSString *query = [url fragment];
    
    if (!query) {
        query = [url query];
    }
    
    NSDictionary* params = [self parseURLParams:query];
    self.mAccessToken = [params valueForKey:@"access_token"];
    
    if (self.mAccessToken != nil) {
        NSLog(@"Instagram Access Token Obtained = %@", self.mAccessToken);
        
        NSString* apiCall = [NSString stringWithFormat:BASE_URL @"/users/self?access_token=%@", self.mAccessToken];
        [self execInstagramAPICall:apiCall andAction:ACTION_GET_USER_SELF withUserInfo:nil];
    }
    else {
        NSString *errorReason = [params valueForKey:@"error"];
        NSLog(@"Instagram Auth ERROR: %@", errorReason);
    }
    return YES;
}

- (void) accessTokenExpired 
{
    [self logout];
}

- (void) execInstagramAPICall:(NSString*)url andAction:(NSString*)_action withUserInfo:(NSDictionary*)_userInfo 
{
    MKNetworkOperation *apiCallOperation = [self operationWithURLString:url params:nil httpMethod:@"GET"];

    [apiCallOperation onCompletion:^(MKNetworkOperation *completedOperation) {   
                            NSLog(@"JSON:%@", [completedOperation responseJSON]);

                            //parse out the json data
                            NSError *error;
                            NSDictionary *json = [NSJSONSerialization 
                                                  JSONObjectWithData:[completedOperation responseData]
                                                             options:kNilOptions 
                                                               error:&error];
        
                            NSDictionary *metaObj = [json objectForKey:@"meta"];
                            if ([[metaObj objectForKey:@"code"] intValue] == 200) {
                                
                                if ([_action isEqualToString:ACTION_GET_USER_SELF]) {
                                    NSDictionary *userJSON = [json objectForKey:@"data"];
                                    if (self.mSessionDelegate) {
                                        [self.mSessionDelegate instagramDidLogin:userJSON];
                                    } 
                                } else if ([_action isEqualToString:ACTION_GET_USER_MEDIA_RECENT]) {
                                    if (self.mRequestDelegate) {
                                        [self.mRequestDelegate instagramResponseReceived:[json objectForKey:@"data"]];
                                    }
                                }
                            }
                      }
                      onError:^(NSError *error) {
                            if (self.mSessionDelegate) {
                                [self.mSessionDelegate instagramDidNotLogin:YES];
                            }
                      }
    ];
    [self enqueueOperation:apiCallOperation];
}

- (void) callUserMediaRecent:(int)count withUserInfo:(NSDictionary*)userInfo {
    NSString* url = [NSString stringWithFormat:BASE_URL @"/users/self/media/recent?access_token=%@&count=%d", self.mAccessToken, count];
    
    [self execInstagramAPICall:url andAction:ACTION_GET_USER_MEDIA_RECENT withUserInfo:userInfo];
}

@end
