
//  Created by Puran Singh on 6/5/12.
//  Copyright (c) 2012 32skills Inc. All rights reserved.
//

#import "InstagramConnectService.h"
#import "InstagramAPIClient.h"
#import "PhotoAlbum.h"
#import "Photo.h"

static NSString *ACCOUNT_ARCHIVE = @"instagramAccount.archive";

@implementation InstagramConnectService

+ (InstagramConnectService *)sharedService {
    static InstagramConnectService *sharedInstagramService = nil;
    if(!sharedInstagramService)
        sharedInstagramService = [[InstagramConnectService alloc] init];
    
    return sharedInstagramService;
}

- (id)initWithAPIClient:(InstagramAPIClient *)client {
    self = [super init];
    if(!self)
        return self;
    instagramClient = client;
    [self loadAccount];
    return self;
}

- (id) init {
    self = [super init];
    if(!self)
        return self;
    
    instagramClient = [[InstagramAPIClient alloc] initWithAppId:INSTAGRAM_CLIENT_ID andDelegate:self];
    [self loadAccount];
    return self;
}

- (void)extendAccessTokenIfNeeded {

}

- (void)albums:(void (^)(NSMutableArray *albums, NSError *error))block
{
    photosCompleteBlock = [block copy];
    NSMutableArray *savedAlbums = [NSKeyedUnarchiver unarchiveObjectWithFile:[self albumsArchivePath]];
    if (savedAlbums) {
       photosCompleteBlock(savedAlbums, nil); 
    } else {
        instagramClient.mRequestDelegate = self;
        [instagramClient callUserMediaRecent:100 withUserInfo:nil];
    }
}

- (BOOL)handleOpenURL:(NSURL *)url {
    return [instagramClient handleOpenURL:url];
}

- (void)saveAccount {
    BOOL archiveSuccessful = [NSKeyedArchiver archiveRootObject:instagramAccount toFile:[self accountArchivePath]];
    if(archiveSuccessful == NO)
        NSLog(@"Couldn't save instagram account");
}

- (void)loadAccount {
    instagramAccount = (NSMutableDictionary *)[NSKeyedUnarchiver unarchiveObjectWithFile:[self accountArchivePath]];
    if(instagramAccount && [instagramAccount objectForKey:@"accessToken"])
        NSLog(@"%@", [instagramAccount objectForKey:@"accessToken"]);
        instagramClient.mAccessToken = [instagramAccount objectForKey:@"accessToken"];
}

- (NSString *)accountArchivePath {
    NSArray *directories = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *directory = [directories objectAtIndex:0];
    return [directory stringByAppendingPathComponent:ACCOUNT_ARCHIVE];
}

#pragma SFAuthenticationService methods

- (BOOL)isLoggedIn {
    return [instagramClient isSessionValid];
}

- (void)loginWithCompletion:(void (^)(id account, NSError *error, BOOL canceled))block
{
    loginCompleteBlock = [block copy];
    if(![instagramClient isSessionValid]) {
        [instagramClient login];
    } else if(loginCompleteBlock) {
        loginCompleteBlock(instagramAccount, nil, NO);
    }
}

- (void)logoutWithCompletion:(void (^)(NSError *error))block {
    logoutCompleteBlock = [block copy];
    [instagramClient logout];
}

- (BOOL)deleteArchive
{
    NSError *error;
    
    if ([[NSFileManager defaultManager] fileExistsAtPath:[self albumsArchivePath]]) {
        [[NSFileManager defaultManager] removeItemAtPath:[self albumsArchivePath] error:&error];
        if (error) {
            return NO;
        }
    }
    return YES;
}


#pragma InstagramAPISessionDelegate methods

- (void) instagramDidLogin:(NSDictionary *) userData {
    NSLog(@"Instagram user data:%@", userData);
    instagramAccount = [[NSMutableDictionary alloc] initWithDictionary:userData];
    [instagramAccount setObject:instagramClient.mAccessToken forKey:@"accessToken"];
    [self saveAccount];
    if(loginCompleteBlock)
        loginCompleteBlock(instagramAccount, nil, NO);
}

- (void) instagramDidNotLogin:(BOOL)cancelled {
    //TODO: fix this when the actual design is implemented
}

- (void) instagramDidLogout {
    //TODO: fix this when the actual design is implemented
}

- (void) instagramSessionInvalidated {
    //TODO: fix this when the actual design is implemented
}

#pragma InstagramAPIRequestDelegate methods

- (void) instagramResponseReceived:(id)jsonData {
    NSMutableArray *photos = [[NSMutableArray alloc] init];
    
    NSArray *dataJSONArray = (NSArray *)jsonData;
    NSLog(@"Instagram image data received");
    for (int i = 0; i < [dataJSONArray count]; i++) {
        Photo *photo = [[Photo alloc] init];
        NSDictionary *photoJSON = [dataJSONArray objectAtIndex:i];
        
        photo.mPhotoID = [photoJSON objectForKey:@"id"];
        photo.mPhotoTitle = [photoJSON objectForKey:@"caption"];
        
        int createdTime = [[photoJSON objectForKey:@"created_time"] intValue];
        if (createdTime != 0) {
            photo.mCreatedDate = [NSDate dateWithTimeIntervalSince1970:createdTime];
        }        
        photo.mSource = INSTAGRAM;
        
        NSDictionary *imagesObj = [photoJSON objectForKey:@"images"];
        
        NSDictionary *standardRes = [imagesObj objectForKey:@"standard_resolution"];
        photo.mFullResImageURL = [NSURL URLWithString:[standardRes objectForKey:@"url"]];
        
        NSDictionary *thumbnail = [imagesObj objectForKey:@"thumbnail"];
        photo.mThumbnailImageURL = [NSURL URLWithString:[thumbnail objectForKey:@"url"]];
        
        [photos addObject:photo];
    }
    NSMutableArray *albums = [[NSMutableArray alloc] init];
    PhotoAlbum *album = [[PhotoAlbum alloc] init];
    album.mAlbumID = @"instagramAlbum";
    album.mAlbumName = @"Instagram Photos";
    album.mSource = INSTAGRAM;
    if ([photos count] > 0) {
        Photo *coverPhoto = [photos objectAtIndex:0];
        album.mCoverPhotoURL = [coverPhoto.mThumbnailImageURL description];
    }
    album.mPhotos = photos;
    
    [albums addObject:album];
    
    BOOL archiveSuccessful = [NSKeyedArchiver archiveRootObject:albums toFile:[self albumsArchivePath]];
    if(archiveSuccessful == NO)
        NSLog(@"Couldn't save instagram albums");
    
    photosCompleteBlock(albums, nil);
}

- (NSString *)albumsArchivePath
{
    NSArray *directories = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *directory = [directories objectAtIndex:0];
    
    return [directory stringByAppendingPathComponent:@"InstagramAlbums.archive"];
}

@end
