//
//  PhotoGridView.m
//  pCarousel
//
//  Created by Puran Singh on 7/4/12.
//  Copyright (c) 2012 32Skills. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>
#import "PhotoGridView.h"
#import "GMGridViewLayoutStrategies.h"
#import "AsyncImageView.h"
#import "PhotoStore.h"

@implementation PhotoGridView

@synthesize delegate;

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self) {        
        NSInteger spacing = 4.8;
        
        NSMutableArray *data = [[NSMutableArray alloc] init];
        _currentData = data;
        
        GMGridView *gmGridView = [[GMGridView alloc] initWithFrame:self.bounds];
        gmGridView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        gmGridView.backgroundColor = [UIColor clearColor];
        [self addSubview:gmGridView];
        
        _gmGridView = gmGridView;
        
        _gmGridView.style = GMGridViewStyleSwap;
        _gmGridView.itemSpacing = spacing;
        _gmGridView.minEdgeInsets = UIEdgeInsetsMake(spacing, spacing, spacing, spacing);
        _gmGridView.centerGrid = NO;
        _gmGridView.actionDelegate = self;
        _gmGridView.transformDelegate = self;
        _gmGridView.dataSource = self;

    }
    return self;
}

//////////////////////////////////////////////////////////////
#pragma mark GMGridViewDataSource
//////////////////////////////////////////////////////////////

- (NSInteger)numberOfItemsInGMGridView:(GMGridView *)gridView
{
    return [_currentData count];
}

- (CGSize)GMGridView:(GMGridView *)gridView sizeForItemsInInterfaceOrientation:(UIInterfaceOrientation)orientation
{
    if (UIInterfaceOrientationIsLandscape(orientation)) 
    {
        return CGSizeMake(74, 74);
    }
    else
    {
        return CGSizeMake(75, 75);
    }
}

- (GMGridViewCell *)GMGridView:(GMGridView *)gridView cellForItemAtIndex:(NSInteger)index
{
    Photo *photo = [_currentData objectAtIndex:index];
    if (shuffle == YES){
        photo = [_currentData objectAtIndex:(([_currentData count] - index) - 1)];
    }
    CGSize size = [self GMGridView:gridView sizeForItemsInInterfaceOrientation:[[UIApplication sharedApplication] statusBarOrientation]];
    
    GMGridViewCell *cell = [gridView dequeueReusableCell];
    
    if (!cell) 
    {
        cell = [[GMGridViewCell alloc] init];        
        UIView *view = [[UIView alloc] initWithFrame:CGRectMake(0, 3, size.width, size.height)];
        view.backgroundColor = [UIColor grayColor];
        view.layer.masksToBounds = NO;
        
        cell.contentView = view;
    }
    
    [[cell.contentView subviews] makeObjectsPerformSelector:@selector(removeFromSuperview)];
    
    AsyncImageView *photoView = [[AsyncImageView alloc] initWithFrame:cell.contentView.bounds];
    photoView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    photoView.mImageType = eDownLoadImageType_THUMBNAIL;
    [photoView setImageWithURL:photo.mThumbnailImageURL onCompletion:^(UIImage *fetchedImage) {
        photoView.image = fetchedImage;
    }];
    [cell.contentView addSubview:photoView];
    
    return cell;
}


- (BOOL)GMGridView:(GMGridView *)gridView canDeleteItemAtIndex:(NSInteger)index
{
    return YES; 
}

//////////////////////////////////////////////////////////////
#pragma mark GMGridViewActionDelegate
//////////////////////////////////////////////////////////////

- (void)GMGridView:(GMGridView *)gridView didTapOnItemAtIndex:(NSInteger)position
{
    NSLog(@"Did tap at index %d", position);
    if (self.delegate) {
        [self.delegate didTapOnItemAtIndex:position];
    }
}

- (void)GMGridViewDidTapOnEmptySpace:(GMGridView *)gridView
{
    NSLog(@"Tap on empty space");
    if (self.delegate) {
        [self.delegate didTaponEmptySpace];
    }
}


//////////////////////////////////////////////////////////////
#pragma mark DraggableGridViewTransformingDelegate
//////////////////////////////////////////////////////////////

- (CGSize)GMGridView:(GMGridView *)gridView sizeInFullSizeForCell:(GMGridViewCell *)cell atIndex:(NSInteger)index inInterfaceOrientation:(UIInterfaceOrientation)orientation
{
    if (UIInterfaceOrientationIsLandscape(orientation)) 
    {
        return CGSizeMake(320, 210);
    }
    else
    {
        return CGSizeMake(300, 310);
    }
}

- (UIView *)GMGridView:(GMGridView *)gridView fullSizeViewForCell:(GMGridViewCell *)cell atIndex:(NSInteger)index
{
    UIView *fullView = [[UIView alloc] init];
    fullView.backgroundColor = [UIColor yellowColor];
    fullView.layer.masksToBounds = NO;
    
    CGSize size = [self GMGridView:gridView sizeInFullSizeForCell:cell atIndex:index inInterfaceOrientation:[[UIApplication sharedApplication] statusBarOrientation]];
    fullView.bounds = CGRectMake(0, 0, size.width, size.height);
    
    UILabel *label = [[UILabel alloc] initWithFrame:fullView.bounds];
    label.text = [NSString stringWithFormat:@"Fullscreen View for cell at index %d", index];
    label.textAlignment = UITextAlignmentCenter;
    label.backgroundColor = [UIColor clearColor];
    label.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    
    label.font = [UIFont boldSystemFontOfSize:15];
    [fullView addSubview:label];
    
    
    return fullView;
}

- (void)GMGridView:(GMGridView *)gridView didStartTransformingCell:(GMGridViewCell *)cell
{
    [UIView animateWithDuration:0.5 
                          delay:0 
                        options:UIViewAnimationOptionAllowUserInteraction 
                     animations:^{
                         cell.contentView.backgroundColor = [UIColor blueColor];
                         cell.contentView.layer.shadowOpacity = 0.7;
                     } 
                     completion:nil];
}

- (void)GMGridView:(GMGridView *)gridView didEndTransformingCell:(GMGridViewCell *)cell
{
    [UIView animateWithDuration:0.5 
                          delay:0 
                        options:UIViewAnimationOptionAllowUserInteraction 
                     animations:^{
                         cell.contentView.backgroundColor = [UIColor redColor];
                         cell.contentView.layer.shadowOpacity = 0;
                     } 
                     completion:nil];
}

- (void)GMGridView:(GMGridView *)gridView didEnterFullSizeForCell:(UIView *)cell
{
    
}

//////////////////////////////////////////////////////////////
#pragma mark private methods
//////////////////////////////////////////////////////////////

- (void)reloadData:(BOOL)sh
{
    self->shuffle = sh;
    _currentData = [[[PhotoStore store] photos] mutableCopy];
    [_gmGridView reloadData];
}

- (UIView *)viewAtIndex:(NSInteger)position
{
    GMGridViewCell *cell = [_gmGridView cellForItemAtIndex:position];
    return cell.contentView;
}
@end
