//
//  PCarouselController.m
//  pCarousel
//
//  Created by Puran Singh on 6/9/12.
//  Copyright (c) 2012 32skills Inc. All rights reserved.
//

#import "PhotoGridViewController.h"
#import "AlbumDataService.h"
#import "PhotoAlbum.h"
#import "AsyncImageView.h"
#import "UIImage+Resize.h"
#import "LoadingView.h"
#import "SlideshowController.h"
#import "PhotoStore.h"
#import "FacebookConnectService.h"
#import "SVProgressHUD.h"

@interface PhotoGridViewController () {
    PhotoStore *photoStore;
}
@property (nonatomic, strong) LoadingView               *mBusyIndicator;
@property (nonatomic, strong) AlbumDataService          *mAlbumDataService;
@property (nonatomic, assign) BOOL shuffle;

- (void)loadPhotos;
- (void)shufflePhotos;
@end

@implementation PhotoGridViewController
@synthesize mBusyIndicator;
@synthesize mAlbumDataService;
@synthesize gridView;
@synthesize shuffle;

- (id)init{
    self = [super init];
    if (self) {
        photoStore = [PhotoStore store];
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    NSLog(@"MEMORY FULL");
}

#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    NSArray* nibViews = [[NSBundle mainBundle] loadNibNamed:@"LoadingView"
                                                      owner:self
                                                    options:nil];
    
    self.mBusyIndicator = [ nibViews objectAtIndex:0];
    self.mBusyIndicator.center = self.view.center;
    self.mBusyIndicator.hidden = YES;
    self.mAlbumDataService = [[AlbumDataService alloc] init];
    
    UITextView *titleView = [[UITextView alloc] initWithFrame:CGRectMake(0, 0, 150, 40)];
    titleView.text = @"Photos";
    titleView.textAlignment = UITextAlignmentCenter;
    titleView.backgroundColor = [UIColor clearColor];
    titleView.textColor = [UIColor whiteColor];
    titleView.font = [UIFont fontWithName:@"American Typewriter"  size:19];
    titleView.editable = NO; 
    
    self.navigationItem.titleView = titleView;
    self.gridView.delegate = self;
    
    UIBarButtonItem *shuffleBtn = [[UIBarButtonItem alloc] initWithTitle:@"Shuffle" style:UIBarButtonItemStyleBordered target:self action:@selector(shufflePhotos)];    
    self.navigationItem.rightBarButtonItem = shuffleBtn;
    
    UIView *texturedBackgroundView = [[UIView alloc] initWithFrame:self.gridView.bounds];
    [texturedBackgroundView setBackgroundColor:[UIColor colorWithPatternImage:[UIImage imageNamed:@"backgroundLeather.png"]]];
    [self.gridView addSubview:texturedBackgroundView];
    [self.gridView sendSubviewToBack:texturedBackgroundView];

    [self.navigationController setToolbarHidden:YES];
    if ([photoStore.activeAlbum.mPhotos count] > 0) {
        [self.gridView reloadData:NO];
    } else {
        [self loadPhotos];
    }
}

- (void)viewDidUnload
{
    [self setGridView:nil];
    [self setMBusyIndicator:nil];
    [self setMAlbumDataService:nil];
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated
{
    [self.navigationController setToolbarHidden:YES];
    [super viewWillAppear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if (UIInterfaceOrientationIsPortrait(self.interfaceOrientation)) {
        return YES;
    }
    return NO;
}

- (void)loadPhotos
{
    [SVProgressHUD showWithStatus:@"Loading Photos..." networkIndicator:YES];
    
    if (photoStore.activeAlbum.mSource == FACEBOOK) {
        
        [[FacebookConnectService sharedService]
         photosFromAlbumWithID:photoStore.activeAlbum.mAlbumID
                    completion:^(NSArray *photos, NSError *error) {
                        [SVProgressHUD dismiss];
                        if (error) {
                            UIAlertView *errorView = [[UIAlertView alloc] initWithTitle:@"ERROR"
                                                                                message:@"Error loading photos"
                                                                               delegate:self cancelButtonTitle:@"OK"
                                                                      otherButtonTitles:nil];
                            [errorView show];
                        } else {
                            [photoStore.activeAlbum.mPhotos addObjectsFromArray:photos];
                            [self.gridView reloadData:NO];
                        }
        }];
        
    } else {
        
        dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0);
        
        dispatch_async(queue, ^{
            [self.mAlbumDataService loadPhotos:photoStore.activeAlbum.mAlbumURL onComplete:^(NSArray *data) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    [SVProgressHUD dismiss];
                    [self.gridView reloadData:NO];
                });
            } onPhoto:^(Photo *photo) {
                [photoStore.activeAlbum.mPhotos addObject:photo];
            } onError:^(NSString *error) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    UIAlertView *errorView = [[UIAlertView alloc] initWithTitle:@"ERROR"
                                                                        message:@"Error loading photos"
                                                                       delegate:self cancelButtonTitle:@"OK"
                                                              otherButtonTitles:nil];
                    [errorView show];
                });
            }];
        });
    }
}

#pragma mark -
#pragma mark GridViewDelegate

- (void)didTapOnItemAtIndex:(NSInteger)position
{
    SlideshowController *slideshowController = [[SlideshowController alloc] init];
    if (self.shuffle == YES) {
        slideshowController.selectedIndex = ([photoStore.activeAlbum.mPhotos count] - 1) - position;
    } else {
        slideshowController.selectedIndex = position;
    }
    [UIView transitionWithView:self.navigationController.view
                      duration:1.0
                       options:UIViewAnimationOptionTransitionCrossDissolve
                    animations:^{
                        [self.navigationController pushViewController:slideshowController animated:NO];
                    }
                    completion:nil];

}

- (void)didTaponEmptySpace
{
}

- (void)shufflePhotos
{
    if (self.shuffle == NO) {
        self.shuffle = YES;
    } else {
        self.shuffle = NO;
    }
    [self.gridView reloadData:self.shuffle];
}

@end

