//
//  PhotoGridView.m
//  pCarousel
//
//  Created by Puran Singh on 7/4/12.
//  Copyright (c) 2012 32Skills. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>
#import "PhotoPageView.h"
#import "AsyncImageView.h"
#import "PhotoStore.h"
#import "SVProgressHUD.h"

@interface PhotoPageView ()
@property (nonatomic, assign)NSInteger *currentIndex;
@end

@implementation PhotoPageView

@synthesize delegate;
@synthesize currentIndex;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
    }
    return self;
}

//////////////////////////////////////////////////////////////
#pragma mark GMGridViewDataSource
//////////////////////////////////////////////////////////////

- (NSInteger)numberOfItemsInGMGridView:(GMGridView *)gridView
{
    return [[PhotoStore store].photos count];
}

- (CGSize)GMGridView:(GMGridView *)gridView sizeForItemsInInterfaceOrientation:(UIInterfaceOrientation)orientation
{
    return self.frame.size;
}

- (GMGridViewCell *)GMGridView:(GMGridView *)gridView cellForItemAtIndex:(NSInteger)index
{
    Photo *photo = [[PhotoStore store].photos objectAtIndex:index];
    CGSize size = [self GMGridView:gridView sizeForItemsInInterfaceOrientation:[[UIApplication sharedApplication] statusBarOrientation]];
    
    GMGridViewCell *cell = [gridView dequeueReusableCell];
    if (!cell)  {
        cell = [[GMGridViewCell alloc] init];
        cell.backgroundColor = [UIColor clearColor];
    }
    else {
        cell.contentView = nil;
    }
    AsyncImageView *photoView = [[AsyncImageView alloc] initWithFrame:CGRectMake(0, 0, size.width, size.height)];
    photoView.mImageType = eDownLoadImageType_LARGE_IMAGE;
    
    __block AsyncImageView *blockView = photoView;
    UIActivityIndicatorView *indicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
    [photoView addSubview:indicator];
    indicator.center = photoView.center;
    [indicator startAnimating];
    [photoView setImageWithURL:photo.mFullResImageURL
                  onCompletion:^(UIImage *fetchedImage) {
                      [indicator stopAnimating];
                      [indicator removeFromSuperview];
                      blockView.image = fetchedImage;
                      CGRect frame = [self getFrameSizeForImage:fetchedImage inImageView:blockView];
                      CGRect imageViewFrame = CGRectMake(blockView.frame.origin.x + frame.origin.x, blockView.frame.origin.y + frame.origin.y, frame.size.width, frame.size.height);
                      blockView.frame = imageViewFrame;
                      photoView.contentMode = UIViewContentModeScaleAspectFit;
    }];
    photoView.contentMode = UIViewContentModeCenter;
    photoView.tag = 5000;
    photoView.backgroundColor = [UIColor clearColor];
    photoView.layer.shadowOffset = CGSizeMake(0.0f, 5.0f);
    photoView.layer.shadowColor = [UIColor blackColor].CGColor;
    photoView.layer.shadowOpacity = 0.8;
    
    cell.contentView = photoView;

    return cell;
}

- (CGRect)getFrameSizeForImage:(UIImage *)image inImageView:(AsyncImageView *)imageView {
    
    float hfactor = image.size.width / imageView.frame.size.width;
    float vfactor = image.size.height / imageView.frame.size.height;
    
    float factor = fmax(hfactor, vfactor);
    
    // Divide the size by the greater of the vertical or horizontal shrinkage factor
    float newWidth = image.size.width / factor;
    float newHeight = image.size.height / factor;
    
    // Then figure out if you need to offset it to center vertically or horizontally
    float leftOffset = (imageView.frame.size.width - newWidth) / 2;
    float topOffset = (imageView.frame.size.height - newHeight) / 2;
    
    return CGRectMake(leftOffset, topOffset, newWidth, newHeight);
}


- (BOOL)GMGridView:(GMGridView *)gridView canDeleteItemAtIndex:(NSInteger)index
{
    return NO; 
}

//////////////////////////////////////////////////////////////
#pragma mark GMGridViewActionDelegate
//////////////////////////////////////////////////////////////

- (void)GMGridView:(GMGridView *)gridView didTapOnItemAtIndex:(NSInteger)position
{
    NSLog(@"Did tap at index %d", position);
    if (self.delegate) {
        [self.delegate didTapOnItemAtIndex:position];
    }
}

- (void)GMGridViewDidTapOnEmptySpace:(GMGridView *)gridView
{
    NSLog(@"Tap on empty space");
    if (self.delegate) {
        [self.delegate didTaponEmptySpace];
    }
}

- (void)GMGridViewDidSwipe:(GMGridView *)gridView direction:(UISwipeGestureRecognizerDirection)direction
{
    if (self.delegate) {
        [self.delegate didSwipe];
    }
}

//////////////////////////////////////////////////////////////
#pragma mark private methods
//////////////////////////////////////////////////////////////

- (void)addMoreItem:(Photo *)photo;
{
    //adding photo at the last position
    [_currentData addObject:photo];
}

- (UIImage *)currentImage
{
    return nil;
}

@end
