//
//  SlideshowController.m
//  pCarousel
//
//  Created by Puran Singh on 7/4/12.
//  Copyright (c) 2012 32Skills Inc. All rights reserved.
//
#import "SlideshowController.h"
#import "AlbumDataService.h"
#import "PhotoStore.h"
#import "LoadingView.h"
#import "AsyncImageView.h"
#import "GMGridViewLayoutStrategies.h"
#import "SVProgressHUD.h"

@interface SlideshowController () {
    AlbumDataService *albumDataService;
}
- (void)editPhoto:(id)sender;
@end

@implementation SlideshowController
@synthesize pageView;
@synthesize selectedIndex;

- (void)didReceiveMemoryWarning
{
    NSLog(@"MEMORY FULL");
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        albumDataService = [[AlbumDataService alloc] init];
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    UITextView *titleView = [[UITextView alloc] initWithFrame:CGRectMake(0, 0, 150, 40)];
    titleView.text = @"Slide Show";
    titleView.textAlignment = UITextAlignmentCenter;
    titleView.backgroundColor = [UIColor clearColor];
    titleView.textColor = [UIColor whiteColor];
    titleView.font = [UIFont fontWithName:@"American Typewriter"  size:19];
    titleView.editable = NO; 
    
    self.navigationItem.titleView = titleView;
    
    UIBarButtonItem *editPhotoBtn = [[UIBarButtonItem alloc] initWithTitle:@"Edit Photo" style:UIBarButtonItemStyleBordered target:self action:@selector(editPhoto:)];
    self.navigationItem.rightBarButtonItem = editPhotoBtn;
    
    self.pageView = [[PhotoPageView alloc] initWithFrame:self.view.bounds];
    
    NSInteger spacing = 0;    
    self.pageView.style = GMGridViewStyleSwap;
    self.pageView.itemSpacing = spacing;
    self.pageView.layoutStrategy = [GMGridViewLayoutStrategyFactory strategyFromType:GMGridViewLayoutHorizontalPagedLTR];
    self.pageView.pagingEnabled = YES;
    self.pageView.minEdgeInsets = UIEdgeInsetsMake(spacing, spacing,spacing, spacing);
    self.pageView.centerGrid = YES;
    self.pageView.actionDelegate = self.pageView;
    self.pageView.dataSource = self.pageView;
    self.pageView.editing = NO;
    self.pageView.delegate = self;
    self.pageView.delegate = self;
    
    [self.view addSubview:self.pageView];
    
    [self.pageView scrollToObjectAtIndex:selectedIndex atScrollPosition:GMGridViewScrollPositionMiddle animated:NO];
    
    UIBarButtonItem *favoriteButton = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"buttonLike.png"]
                                                                       style:UIBarButtonItemStylePlain
                                                                      target:self
                                                                      action:@selector(markFavorite)];
    [favoriteButton setBackButtonBackgroundImage:[UIImage imageNamed:@"buttonLikeSelected.png"]
                                        forState:UIControlStateHighlighted
                                      barMetrics:UIBarMetricsDefault];
    
    UIBarButtonItem *flexible = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    UIBarButtonItem *actionButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAction
                                                                                  target:self action:@selector(photoAction:)];
    
    self.toolbarItems = [NSArray arrayWithObjects:favoriteButton, flexible, actionButton, nil];
    
    self.navigationController.toolbar.barStyle = UIBarStyleBlackTranslucent;
    self.navigationController.navigationBar.barStyle = UIBarStyleBlackTranslucent;
}

- (void)viewDidUnload
{
    [self setPageView:nil];
    self->albumDataService = nil;
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated
{
    self.navigationController.navigationBar.translucent=YES;
    [self.navigationController setToolbarHidden:YES];
    [self.navigationController setNavigationBarHidden:YES];
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    self.navigationController.navigationBar.barStyle = UIBarStyleDefault;
        self.navigationController.navigationBar.translucent=NO;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if (UIInterfaceOrientationIsPortrait(self.interfaceOrientation)) {
        return YES;
    }
    return NO;
}

- (void)photoEditor:(AFPhotoEditorController *)editor finishedWithImage:(UIImage *)image
{
    [self dismissModalViewControllerAnimated:YES];
}

- (void)photoEditorCanceled:(AFPhotoEditorController *)editor
{
    [self dismissModalViewControllerAnimated:YES];
}

- (void)loadPhotoEditor:(UIImage *)image
{
    AFPhotoEditorController *editor = [[AFPhotoEditorController alloc] initWithImage:image];
    
    [editor setDelegate:self];
    UIColor *backgroundColor = [UIColor colorWithRed:122.0f/255.0f green:204.0f/255.0f blue:41.0f/255.0f alpha:1.0];
    UIColor *foregroundColor = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:1.0];
    
    AFPhotoEditorStyle *style = [editor style];
    [style setTopBarBackgroundColor:backgroundColor];
    [style setTopBarTextColor:foregroundColor];
    
    [self presentViewController:editor animated:YES completion:nil];
}

- (void)editPhoto:(id)sender
{
    NSArray* nibViews = [[NSBundle mainBundle] loadNibNamed:@"LoadingView"
                                                      owner:self
                                                    options:nil];
    
    LoadingView *busy = [nibViews objectAtIndex:0];
    [self.view addSubview:busy];
    busy.center = self.view.center;

    [busy start];
    
    Photo *photo = [[[PhotoStore store] photos] objectAtIndex:self.selectedIndex];
    if (photo.mSource == IPHONE) {
        [albumDataService loadImage:photo.mFullResImageURL imageBlock:^(UIImage *image) {
            [busy stop];
            [self loadPhotoEditor:image];
        }];
    } else {
        AsyncImageView *loadImageView = [[AsyncImageView alloc] init];
        
        [loadImageView setImageWithURL:photo.mFullResImageURL
                          onCompletion:^(UIImage *fetchedImage) {
            [busy stop];
            NSLog(@"%f, %f", fetchedImage.size.width, fetchedImage.size.height);
            [self loadPhotoEditor:fetchedImage];
        }];
    }
}

- (void)didTapOnItemAtIndex:(NSInteger)position
{
    self.selectedIndex = position;
    
    [self.navigationController setToolbarHidden:NO animated:YES];
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    

}

- (void)didTaponEmptySpace
{
    [self.navigationController setToolbarHidden:NO animated:YES];
    [self.navigationController setNavigationBarHidden:NO animated:YES];
}

- (void)didSwipe
{
    [self.navigationController setToolbarHidden:YES animated:YES];
    [self.navigationController setNavigationBarHidden:YES animated:YES];
}
 
- (void)markFavorite
{
    NSLog(@"Selected index:%d", self.selectedIndex);
    Photo *photo = [[[PhotoStore store] photos] objectAtIndex:self.selectedIndex];
    
    [albumDataService addToFavorite:photo.mPhotoID photo:photo];
    [SVProgressHUD show];
    [SVProgressHUD dismissWithSuccess:@"Photo added favorite list"];
}
@end